<?php

declare(strict_types=1);

namespace Drupal\mautic_api;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\mautic_api\Entity\MauticApiConnection;
use Mautic\Api\Api;
use Mautic\Auth\ApiAuth;
use Mautic\Auth\TwoLeggedOAuth2;
use Mautic\Exception\IncorrectParametersReturnedException;
use Mautic\Exception\RequiredParameterMissingException;
use Mautic\MauticApi;
use Symfony\Component\HttpFoundation\RequestStack;
use Psr\Log\LoggerInterface;
use Drupal\Core\State\StateInterface;

/**
 * Service for handling Mautic API connections and requests.
 */
final class MauticApiConnector {

  /**
   * The Mautic API factory.
   */
  protected MauticApi $mauticApi;

  /**
   * Constructs a new MauticApiConnector object.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected RequestStack $requestStack,
    protected LoggerInterface $logger,
    protected StateInterface $state,
  ) {
    $this->mauticApi = new MauticApi();
  }

  /**
   * Makes a request to the Mautic API.
   *
   * @param string $connection_id
   *   The Mautic API connection configuration entity ID.
   * @param string $endpoint
   *   The API endpoints like contacts, forms, and segments.
   * @param int|null $id
   *   ID of the contact, form, or segment.
   * @param string $action
   *   Actions such as Create, Update, or Delete.
   * @param array $data
   *   Optional data to send with the request.
   *
   * @return array|null
   *   The API response data or NULL if the request failed.
   */
  public function request(string $connection_id, string $endpoint, ?int $id = NULL, string $action = '', array $data = []): ?array {
    try {
      $connection = $this->entityTypeManager
        ->getStorage('mautic_api_connection')
        ->load($connection_id);

      if (!$connection instanceof MauticApiConnection) {
        throw new \Exception("Mautic API connection with ID {$connection_id} not found.");
      }

      $settings = $this->prepSettings($connection);
      $api = $this->initializeApiClient($settings, $connection->getAuthMethod(), $endpoint, $connection_id);

      switch ($action) {
        case 'list':
          return isset($data['search']) ? $api->getList($data['search']) : $api->getList();

        case 'create':
          if ($id === NULL && isset($data)) {
            return $api->create($data);
          }
          throw new RequiredParameterMissingException("Create action requires a valid set of data.");

        case 'edit':
          if ($id !== NULL) {
            return $api->edit($id, $data);
          }
          throw new RequiredParameterMissingException("Edit action requires a valid ID");

        case 'delete':
          if ($id !== NULL) {
            return $api->delete($id);
          }
          throw new RequiredParameterMissingException("Delete action requires a valid ID");

        case 'get':
          if ($id !== NULL) {
            return $api->get($id);
          }
          throw new RequiredParameterMissingException("Get action requires an ID");

        case 'contact-to-list':
          if ($id !== NULL && isset($data['contact_id'])) {
            /** @var \Mautic\Api\Segments $api */
            return $api->addContact($id, $data['contact_id']);
          }
          throw new RequiredParameterMissingException("Segment ID or Contact ID not found.");

        case 'add-contact-dnc':
          if ($id !== NULL) {
            /** @var \Mautic\Api\Contacts $api */
            return $api->addDnc($id);
          }
          throw new RequiredParameterMissingException("Contact ID not found to add as DNC.");

        case 'remove-contact-dnc':
          if ($id !== NULL) {
            /** @var \Mautic\Api\Contacts $api */
            return $api->removeDnc($id);
          }
          throw new RequiredParameterMissingException("Contact ID not found to remove as DNC.");

        default:
          throw new IncorrectParametersReturnedException("Invalid action '{$action}'");
      }
    }
    catch (\Exception $e) {
      $this->logger->error('Mautic API request failed: @error', ['@error' => $e->getMessage()]);
      throw new \RuntimeException("Failed to execute Mautic API request: {$e->getMessage()}", 0, $e);
    }
  }

  /**
   * Get the status of the Mautic API connection.
   *
   * @param \Drupal\mautic_api\Entity\MauticApiConnection $connection
   *   The Mautic API connection entity.
   * @param string $endpoint
   *   The API endpoint to test (e.g., 'contacts', 'forms', 'segments').
   *
   * @return array
   *   An array containing:
   *   - status_code: The HTTP status code if the request was successful,
   *     NULL otherwise
   *   - message: A descriptive message about the status or error
   */
  public function getStatus(MauticApiConnection $connection, string $endpoint): array {
    $settings = $this->prepSettings($connection);
    $api = $this->initializeApiClient($settings, $connection->getAuthMethod(), $endpoint, $connection->id());

    try {
      $response = $api->getList('', 0, 1, '', 'ASC', FALSE, TRUE);
      if ($response) {
        $responseInfo = $api->getResponseInfo();
        $statusCode = $responseInfo['http_code'] ?? NULL;
        if ($statusCode === 200) {
          $message = "Successfully connected to the Mautic API instance. <b>STATUS: 200 OK</b.";
        }
        else {
          $message = "Mautic API connection <b>FAILED</b>. Please check the logs for more information.";
        }
      }
      else {
        $message = "<b>NO RESPONSE RECEIVED</b> from the Mautic API. Please check your connection settings.";
      }
    }
    catch (\Exception $e) {
      $message = $e->getMessage();
      $this->logger->error('Mautic API status check failed: @error', ['@error' => $message]);
    }

    return [
      'status_code' => $statusCode,
      'message' => $message,
    ];
  }

  /**
   * Prepare the settings for the Mautic auth objects.
   *
   * @param \Drupal\mautic_api\Entity\MauticApiConnection $connection
   *   The Mautic API connection entity.
   *
   * @return array
   *   The prepared settings array.
   */
  protected function prepSettings(MauticApiConnection $connection): array {
    $settings = [
      'baseUrl' => $this->getCompleteUrl($connection),
    ];

    if ($connection->getAuthMethod() === 'oauth') {
      $settings += [
        'version'          => 'OAuth2',
        'clientKey'        => $connection->getPublicKey(),
        'clientSecret'     => $connection->getSecretKey(),
        'callback' => $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost() . $this->requestStack->getCurrentRequest()->getRequestUri(),
      ];
    }
    else {
      $settings += [
        'userName'   => $connection->getBasicAuthUsername(),
        'password'   => $connection->getBasicAuthPassword(),
      ];
    }
    return $settings;
  }

  /**
   * Initialize the connection.
   *
   * @param array $settings
   *   The settings array.
   * @param string $authMethod
   *   The authentication method.
   * @param string $endpoint
   *   The API endpoint to use.
   * @param string $connection_id
   *   The connection ID.
   *
   * @return \Mautic\Api\Api
   *   The initialized API client instance for the specific endpoint.
   */
  protected function initializeApiClient(array $settings, string $authMethod, string $endpoint, string $connection_id): Api {
    try {
      $initAuth = new ApiAuth();

      if ($authMethod === 'oauth') {
        /** @var \Mautic\Auth\TwoLeggedOAuth2 $auth */
        $auth = $initAuth->newAuth($settings, 'TwoLeggedOAuth2');

        $token_status = $this->getOauthAccessToken($auth, $connection_id);
        if (!$token_status['success']) {
          $this->logger->error($token_status['message']);
        }

      }
      else {
        $auth = $initAuth->newAuth($settings, 'BasicAuth');
      }

      return $this->mauticApi->newApi($endpoint, $auth, $settings['baseUrl']);
    }
    catch (\Exception $e) {
      $this->logger->error('API client initialization failed: @error', ['@error' => $e->getMessage()]);
      throw new \RuntimeException("Failed to initialize API client: {$e->getMessage()}", 0, $e);
    }
  }

  /**
   * Get and manage OAuth access token for a specific connection.
   *
   * @param \Mautic\Auth\TwoLeggedOAuth2 $auth
   *   The TwoLeggedOAuth2 auth object.
   * @param string $connection_id
   *   The Mautic API connection configuration entity ID.
   *
   * @return array
   *   The status of the access token operation.
   */
  public function getOauthAccessToken(TwoLeggedOAuth2 $auth, string $connection_id): array {
    $state_key = 'mautic_api.oauth_access_tokens';
    $tokens = $this->state->get($state_key, []);

    if (!empty($tokens[$connection_id])) {
      $auth->setAccessTokenDetails($tokens[$connection_id]);
    }

    if (!$auth->validateAccessToken()) {
      $auth->setAccessTokenDetails(['access_token' => '']);
      try {
        $auth->requestAccessToken();
        $accessTokenData = $auth->getAccessTokenData();

        $tokens[$connection_id] = $accessTokenData;
        $this->state->set($state_key, $tokens);

        return [
          'success' => TRUE,
          'message' => 'Successfully obtained access token.',
        ];
      }
      catch (\Exception $e) {
        $error_message = $e->getMessage();
        if (strpos($error_message, 'invalid_client') !== FALSE) {
          return [
            'success' => FALSE,
            'message' => 'Failed to obtain the OAuth Access Token. Please verify your client ID and client secret.',
          ];
        }
        return [
          'success' => FALSE,
          'message' => 'Failed to obtain Mautic API access token: ' . $error_message,
        ];
      }
    }

    return [
      'success' => TRUE,
      'message' => 'Using existing valid access token.',
    ];
  }

  /**
   * Remove OAuth access token for a specific connection.
   *
   * @param string $connection_id
   *   The Mautic API connection configuration entity ID.
   */
  public function removeOauthAccessToken(string $connection_id): void {
    $state_key = 'mautic_api.oauth_access_tokens';
    $tokens = $this->state->get($state_key, []);

    if (isset($tokens[$connection_id])) {
      unset($tokens[$connection_id]);
      $this->state->set($state_key, $tokens);
      $this->logger->info('Cleared OAuth access token for Mautic API connection: @id', ['@id' => $connection_id]);
    }
  }

  /**
   * Get the complete URL for a Mautic connection.
   *
   * @param \Drupal\mautic_api\Entity\MauticApiConnection $connection
   *   The Mautic API Connection object.
   *
   * @return string
   *   The complete URL for the connection.
   */
  public function getCompleteUrl(MauticApiConnection $connection): string {
    $base_url = $connection->getHttpProtocol() . '://' . $connection->getBaseUrl();

    if ($connection->getPort()) {
      $base_url .= ':' . $connection->getPort();
    }

    if ($connection->getPath()) {
      $base_url .= '/' . trim($connection->getPath(), '/');
    }

    return $base_url;
  }

}
